    
    // LinkedList class
    //
    // CONSTRUCTION:
    //
    // ******************PUBLIC OPERATIONS*********************
    // boolean isEmpty( )     --> Restituisce true se la lista  vuota; false altrimenti
    // void clear( )          --> Svuota la lista
    // int getSize( )         --> Restituisce la dimensione effettiva della lista
    // ListNode getHead( )    --> Restituisce la testa della lista
    // void insertHead( x )   --> Inserisce x in testa alla lista
    // void insertTail( x )   --> Inserisce x in coda alla lista
    // void insertOrd(Object x)--> Inserimento ordinato di x
    // ListNode searchOrd( x ) --> Ricerca ordinata di x
    // ListNode search( x )    --> Ricerca esaustiva di x
    // Object deleteHead( )    --> Cancella l'elemento in testa e restituisce l'info
    // Object deleteTail( )    --> Cancella l'elemento in coda e restituisce l'info
    // ListNode deleteKey( x ) --> Cancella e restituisce il nodo di chiave x
    // LinkedList concat(LinkedList l) --> Concatena la lista l alla lista in questione
    // ******************ERRORS********************************
    // Nessuno

   
    public class LinkedList
    {

       private ListNode head;
       private int size = 0;

        /**
         * Costruisce una lista
         */
        public LinkedList( )
        {
            head =  null;
        }

        /**
         * Test se la lista  vuota.
         * @return true se vuota, false altrimenti.
         */
        public boolean isEmpty( )
        {
            return head == null;
        }

        /**
         * Svuota la lista.
         */
        public void clear( )
        {
            head = null;
            size = 0;
        }

        /**
        * Restituisce il numero effettivo di elementi nella lista.
        */
       public int getSize( )
       {
           return size;
       }



        /**
         * Restituisce la testa della lista.
         */
        public ListNode getHead( )
        {
            return head;
        }


        /**
         * Setta la testa della lista ad un nuovo valore.
         */
        public void setHead( ListNode h )
        {
            head = h;
        }


        /**
         * Inserisce un elemento in testa alla lista.
         */
         public void insertHead(Object x)
         {
            head = new ListNode(x,head);
            size++;
         }


         /**
          * Inserisce un elemento in coda alla lista.
          */
          public void insertTail(Object x)
          {
             if (isEmpty())
               head = new ListNode(x);
             else
             {
               ListNode aux = head;
               for(; aux.next!= null; aux = aux.next);
               aux.next = new ListNode(x);
             }
             size++;
          }

          /**
           * Concatena una data lista alla lista in questione.
           */
              public void concat(LinkedList l)
              {
                 if (!l.isEmpty() && !this.isEmpty())
                  {
                    ListNode aux = head;
                    for(; aux.next!= null; aux = aux.next)
                      ;
                    aux.next = l.getHead();
                    size += l.getSize();
                    }
                if (!l.isEmpty() && this.isEmpty()){
                   head = l.getHead();
                   size = l.getSize();
                 }

              }


         /**
          * Inserimento ordinato di un elemento. Per il confronto viene fatto
          * il cast esplicito dell'oggetto info nel tipo Comparable.
          * @param x l'elemento da inserire.
          * @see java.lang.Comparable
          */
         public void insertOrd( Object x )
         {
           if (isEmpty())
           {
             head = new ListNode( x );
             size++;
           }
           else
           {
             if ( ((Comparable) head.info).compareTo(x) >= 0 )
               insertHead ( x );
             else
             {
               ListNode aux = head;
               for(; ( aux.next != null ) && 
		     ( ((Comparable)aux.next.info).compareTo(x) < 0 );
                      aux = aux.next );
               aux.next = new ListNode(x, aux.next);
               size++;
             }
           }

         }


        /**
         * Restituisce il nodo contenente la prima occorrenza dell'elemento di chiave x.
         * Per il confronto viene fatto il cast esplicito dell'oggetto info nel tipo Comparable.
         * @param key la chiave dell'elemento da cercare.
         * @return il ListNode contenente l'elemento di chiave x, null altrimenti.
         * @see java.lang.Comparable
         */
         public ListNode searchOrd( Object x )
         {
           ListNode aux = head;
           for( ; aux != null && ((Comparable)aux.info).compareTo(x) < 0; aux = aux.next);
           if (aux != null && aux.info.equals(x))
             return aux;

           return null;
         }


        /**
         * Restituisce il nodo contenente la prima occorrenza dell'elemento di chiave x.
         * @param key la chiave dell'elemento da cercare.
         * @return il ListNode contenente l'elemento di chiave x, null altrimenti.
         */
        public ListNode search( Object x )
        {
          ListNode aux = head;
          for( ; aux != null && !aux.info.equals( x ); aux = aux.next);
          return aux;
        }


        /**
         * Cancella l'elemento in testa e restituisce l'info.
         */
        public Object deleteHead()
        {
          if (isEmpty())
              return null;

          ListNode aux = head;
          head = head.next;
          size--;

          return aux.info;
        }


        /**
         * Cancella l'elemento in coda e restituisce l'info.
         */
        public Object deleteTail()
        {
          if (isEmpty())
              return null;

          ListNode aux = head;
          ListNode pred = null;
          for(; aux.next != null; pred = aux, aux = aux.next);
          if (pred == null)
             head = null;
          else
             pred.next = null;
          size--;

          return aux.info;

        }


        /**
         * Cancella e restituisce il nodo di chiave x.
         */
         public ListNode deleteKey(Object x)
         {
             ListNode aux = head;
             ListNode pred = null;
             for(; aux != null && !aux.info.equals(x); pred = aux, aux = aux.next);
             if (aux != null){
                if (pred == null)
                  head = head.next;
                else
                  pred.next = aux.next;
                size--;
             }
             return aux;
          }

         /**
         * Cancella e restituisce il nodo alla posizione pos
         * (iniziando a contare da 0, 1, 2, 3 ...).
         */
         public ListNode deleteAt(int pos){

           ListNode aux = head;
           ListNode pred = null;

          for(int count = 0; aux != null && count != pos;
             pred = aux, aux = aux.next, count++);

          if (aux != null){
           if (pred == null)
               head = head.next;
           else pred.next = aux.next;
           size --;
          }
         return aux;

        }

  //Un semplice metodo di stampa
        public void printList( )
        {
            if( this.isEmpty( ) )
                System.out.print( "Empty list" );
            else
            {
                ListNode aux = this.head;
                for(; aux != null; aux = aux.next)
                    System.out.print( aux.info + " " );
            }

            System.out.println("\nSize = "+ this.size + " \n" );
        }

    }