/**
* Wrapper class per uso generico come involucro di interi.
* Mima la classe Integer ed implementa l'interfaccia Comparable.
* Da Java 1.2, si pu usare direttamente la classe Integer se  richiesto
* l'uso dell'interfaccia Comparable.
* @see java.lang.Comparable
* @author E. Riccobene, S. Riccobene, P. Scandurra
*/

public final class MyInteger implements Comparable{
      private int value;

        /**
         * Construisce un oggetto MyInteger con valore iniziale 0.
         */
        public MyInteger( ){ this( 0 ); }

        /**
         * Costruisce un oggetto MyInteger.
         * @param x il valore iniziale.
         */
        public MyInteger( int x ){ value = x; }

        /**
         * Restituisce il valore intero memorizzato.
         * @return il valore memorizzato.
         */
        public int intValue( ){ return value; }

        /**
         * Implementazione del metodo toString.
         * @return la rappresentazione String dell'oggetto.
         */
        public String toString( ) {
            return Integer.toString( value );
        }

        /**
         * Implementazione del metodo compareTo.
         * @param rhs l'altro oggetto MyInteger per il confronto.
         * @return 0 se i due oggetti sono uguali; un intero < 0
         * se l'oggetto rhs  pi grande; un intero > 0 se rhs  pi piccolo.
         * @exception ClassCastException se rhs non  di tipo MyInteger.
         */
        public int compareTo( Object rhs ) {
            return value < ((MyInteger)rhs).value ? -1 : 
	      		value == ((MyInteger)rhs).value ? 0 : 1;
        }

        /**
         * Implementazione del metodo equals.
         * @param rhs il secondo oggetto MyInteger per il confronto.
         * @return true se i due oggetti sono uguali, false altrimenti.
         * @exception ClassCastException se rhs non  di tipo MyInteger.
         */
        public boolean equals( Object rhs ) {
            return rhs != null && value == ((MyInteger)rhs).value;
        }

    }
